@@ -18,7 +18,8 @@ import { LLMStateDisplay } from './LLMStateDisplay/LLMStateDisplay';
 import {
   HANDS_ON_MODE_ICON,
   ASK_ICON,
-  MODE_ICON,
+  AGENT_MODE_ICON,
+  AGENT_MODE_SHINY_ICON,
   OPEN_MODE_SELECTOR_ICON,
   SEND_ICON
 } from './icons';
@@ -384,7 +385,7 @@ export class ChatBoxWidget extends Widget {
     const agentOption = this.createOption(
       'agent',
       'Agent',
-      MODE_ICON.svgstr,
+      AGENT_MODE_ICON.svgstr,
       'Prepare datasets. Build models. Test ideas.'
     );
     const askOption = this.createOption(
@@ -411,6 +412,9 @@ export class ChatBoxWidget extends Widget {
     // Add click handler to toggle dropdown
     this.modeSelector.addEventListener('click', e => {
       e.stopPropagation();
+      if (this.modeSelector.getAttribute('data-is-disabled') === 'true') {
+        return;
+      }
       this.toggleModeDropdown();
     });
 
@@ -755,14 +759,10 @@ export class ChatBoxWidget extends Widget {
       const displayOption = document.createElement('div');
       displayOption.className = 'sage-ai-mode-display';
 
-      // Clone only the icon from the selected option
-      const originalIcon = selectedOption.querySelector(
-        '.sage-ai-mode-option-icon'
-      );
-      if (originalIcon) {
-        const iconContainer = originalIcon.cloneNode(true) as HTMLDivElement;
-        displayOption.appendChild(iconContainer);
-      }
+      const iconContainer = document.createElement('div');
+      iconContainer.className = 'sage-ai-mode-option-icon';
+      iconContainer.innerHTML = AGENT_MODE_SHINY_ICON.svgstr;
+      displayOption.appendChild(iconContainer);
 
       // Create text element with only the title (no description)
       const originalText = selectedOption.querySelector(
@@ -1104,7 +1104,7 @@ export class ChatBoxWidget extends Widget {
     this.uiHelper.updateAgentModeElement(this.modeSelector, false);
 
     const modeIcon = document.createElement('div');
-    MODE_ICON.render(modeIcon);
+    AGENT_MODE_SHINY_ICON.render(modeIcon);
 
     // Check if there are pending diffs and show approval dialog if needed
     const diffManager = AppStateService.getState().notebookDiffManager;
