@@ -1,8 +1,10 @@
+import { parse as parseBestEffortJson } from 'best-effort-json-parser';
 import { INotebookTracker, NotebookPanel } from '@jupyterlab/notebook';
 import { NotebookContextManager } from './NotebookContextManager';
 import { Cell } from '@jupyterlab/cells';
 import { NotebookTools } from './NotebookTools';
 import { NotebookDiffTools } from './NotebookDiffTools';
+import { AppStateService } from '../AppState';
 
 /**
  * Service that highlights cells that are in context
@@ -15,18 +17,15 @@ export class ContextCellHighlighter {
   private highlightedCells: Map<string, Set<string>> = new Map(); // Map of notebook path to set of highlighted cell IDs
   private chatContainerRef: any = null; // Reference to chat container for updates
   private cellHistory: Map<string, string[]> = new Map(); // Map of cell tracking ID to array of content versions for undo
-  private onPromptSubmit: (cell: Cell, prompt: string) => Promise<boolean>; // Callback for prompt submission
 
   constructor(
     notebookTracker: INotebookTracker,
     notebookContextManager: NotebookContextManager,
-    notebookTools: NotebookTools,
-    onPromptSubmit: (cell: Cell, prompt: string) => Promise<boolean>
+    notebookTools: NotebookTools
   ) {
     this.notebookTracker = notebookTracker;
     this.notebookContextManager = notebookContextManager;
     this.notebookTools = notebookTools;
-    this.onPromptSubmit = onPromptSubmit; // Store the callback
 
     // Add CSS for context highlighting
     this.addContextHighlightCSS();
@@ -34,6 +33,429 @@ export class ContextCellHighlighter {
     this.setupListeners();
   }
 
+  /**
+   * Handle prompt submission for AI quick generation
+   */
+  private async onPromptSubmit(
+    cell: Cell,
+    promptText: string
+  ): Promise<boolean> {
+    // This is the callback executed when a prompt is submitted from the quick generation input
+    console.log(
+      'Prompt submitted for cell',
+      (cell as any).model?.id || cell.id,
+      ':',
+      promptText
+    );
+
+    // Get the cell ID (use tracking ID if available, fallback to model.id or id)
+    const cellId =
+      (cell as any).model?.sharedModel.getMetadata()?.cell_tracker
+        ?.trackingId ||
+      (cell as any).model?.id ||
+      (cell as any).id ||
+      '[unknown]';
+
+    if (cellId === '[unknown]') {
+      console.error('Could not determine cell ID for prompt submission.', cell);
+      return false;
+    }
+
+    const activeCell = this.notebookTracker.activeCell;
+    if (!activeCell) {
+      console.warn('No active cell');
+      return false;
+    }
+
+    const editor = activeCell.editor;
+    const selection = editor?.getSelection();
+    const cellLines = editor?.model.sharedModel.source.split('\n') || [];
+    const endSelection = this.calculateEndLine(selection?.end, cellLines);
+    const selectedText = editor?.model.sharedModel.source.substring(
+      editor.getOffsetAt(selection?.start || { line: 0, column: 0 }),
+      editor.getOffsetAt(endSelection)
+    );
+
+    // Check if there's selected text to determine the mode
+    const hasSelection = selectedText && selectedText.trim().length > 0;
+
+    // For edit_selection, get the selected line range
+    const startLine = selection?.start?.line || 0;
+    const endLine = endSelection.line;
+    const selectedLines = cellLines.slice(startLine, endLine + 1);
+
+    const config = AppStateService.getConfig();
+
+    // Determine which config section to use based on whether text is selected
+    let modeConfig;
+    let modeLabel;
+    if (hasSelection) {
+      modeConfig = config.edit_selection;
+      modeLabel = 'edit_selection';
+    } else {
+      modeConfig = config.edit_full_cell;
+      modeLabel = 'edit_full_cell';
+    }
+
+    const systemPrompt = modeConfig.system_prompt;
+
+    // Use ephemeral API call instead of chatbox for faster response
+    try {
+      const targetCell = this.notebookTools.findCellByTrackingId(cellId);
+      const cellContent = targetCell?.cell.model.sharedModel.getSource();
+      const previousCellContent = this.notebookTools
+        .findCellByIndex((targetCell?.index || 0) - 1)
+        ?.cell?.model?.sharedModel.getSource();
+
+      // Compose enhanced context messages based on whether text is selected
+      let contextMessage = '';
+
+      if (hasSelection) {
+        // Create line-numbered cell content for the selection range
+        const selectionRangeLines = selectedLines
+          .map((line, index) => {
+            const lineNum = startLine + index + 1; // Use actual line numbers from the cell
+            return `${lineNum.toString().padStart(3, ' ')}: ${line}`;
+          })
+          .join('\n');
+
+        // Create line-numbered full cell content for context
+        const fullCellLines = cellLines
+          .map((line, index) => {
+            const lineNum = index + 1;
+            return `${lineNum.toString().padStart(3, ' ')}:${line}`;
+          })
+          .join('\n');
+
+        contextMessage = `EDIT SELECTION MODE: Edit the selected lines (${startLine + 1}-${endLine + 1}) using structured operations.
+
+FULL CELL CONTEXT:
+\`\`\`
+${fullCellLines}
+\`\`\`
+
+SELECTED LINES TO EDIT (${startLine + 1}-${endLine + 1}):
+\`\`\`
+${selectionRangeLines}
+\`\`\`
+
+Return a JSON object with operations for each line in the selection range. Use KEEP, MODIFY, REMOVE, or INSERT actions. Consider the full cell context when making edits.
+
+User request: ${promptText}`;
+      } else {
+        contextMessage = `
+              EDIT FULL CELL MODE: Edit and improve the complete cell.
+
+              CELL TO EDIT:
+              \`\`\`
+              ${cellContent}
+              \`\`\`
+
+              PREVIOUS CELL CONTENT:
+              \`\`\`
+              ${previousCellContent}
+              \`\`\`
+
+              Return only the fully edited cell. Apply your quantitative expertise to improve the code.
+
+              User request: ${promptText}`;
+      }
+
+      const chatService = AppStateService.getChatService();
+
+      // Handle streaming based on whether text is selected
+      if (hasSelection) {
+        // For edit_selection, use structured JSON operations with streaming
+        const targetCell = this.notebookTools.findCellByTrackingId(cellId);
+        if (!targetCell) {
+          throw new Error(`Could not find cell with ID ${cellId}`);
+        }
+
+        let accumulatedResponse = '';
+        let lastAppliedOperations: EditOperation[] = [];
+
+        // Helper function to try parsing JSON and extract complete operations
+        const tryParseAndApplyOperations = (text: string): EditOperation[] => {
+          try {
+            // Use best-effort JSON parser to handle incomplete JSON
+            const parsedResponse = parseBestEffortJson(
+              text.trim()
+            ) as EditSelectionResponse;
+
+            if (
+              parsedResponse?.operations &&
+              Array.isArray(parsedResponse.operations)
+            ) {
+              return parsedResponse.operations.filter(
+                (op: EditOperation) =>
+                  op.line !== undefined &&
+                  op.action &&
+                  ['KEEP', 'MODIFY', 'REMOVE', 'INSERT'].includes(op.action)
+              );
+            }
+          } catch (error) {
+            // JSON parsing failed, return empty array
+          }
+          return [];
+        };
+
+        // Helper function to apply operations progressively
+        const applyOperationsToCell = (operations: EditOperation[]) => {
+          if (operations.length === 0) return;
+
+          const updatedCellLines = [...cellLines];
+          let lineOffset = 0;
+
+          // Sort operations by line number to process in order
+          const sortedOperations = operations.sort(
+            (a: EditOperation, b: EditOperation) => a.line - b.line
+          );
+
+          for (const operation of sortedOperations) {
+            const actualLineIndex = operation.line - 1 + lineOffset;
+
+            switch (operation.action) {
+              case 'KEEP':
+                // Do nothing - line stays as is
+                break;
+              case 'MODIFY':
+                if (
+                  actualLineIndex < updatedCellLines.length &&
+                  operation.content !== undefined
+                ) {
+                  updatedCellLines[actualLineIndex] = operation.content;
+                }
+                break;
+              case 'REMOVE':
+                if (actualLineIndex < updatedCellLines.length) {
+                  updatedCellLines.splice(actualLineIndex, 1);
+                  lineOffset--;
+                }
+                break;
+              case 'INSERT':
+                if (
+                  actualLineIndex <= updatedCellLines.length &&
+                  operation.content !== undefined
+                ) {
+                  updatedCellLines.splice(
+                    actualLineIndex,
+                    0,
+                    operation.content
+                  );
+                  lineOffset++;
+                }
+                break;
+            }
+          }
+
+          // Apply the changes to the cell
+          targetCell.cell.model.sharedModel.setSource(
+            updatedCellLines.join('\n')
+          );
+        };
+
+        const response = await chatService.sendEphemeralMessage(
+          contextMessage,
+          systemPrompt,
+          'claude-3-5-haiku-latest',
+          (textChunk: string) => {
+            accumulatedResponse += textChunk;
+
+            // Try to parse and apply operations from the accumulated response
+            const currentOperations =
+              tryParseAndApplyOperations(accumulatedResponse);
+
+            // Only apply if we have new or changed operations
+            if (currentOperations.length > 0) {
+              const operationsChanged =
+                currentOperations.length !== lastAppliedOperations.length ||
+                currentOperations.some((op, idx) => {
+                  const lastOp = lastAppliedOperations[idx];
+                  return (
+                    !lastOp ||
+                    op.line !== lastOp.line ||
+                    op.action !== lastOp.action ||
+                    op.content !== lastOp.content
+                  );
+                });
+
+              if (operationsChanged) {
+                applyOperationsToCell(currentOperations);
+                lastAppliedOperations = [...currentOperations];
+              }
+            }
+          }
+        );
+
+        // Final parsing and application
+        try {
+          const finalOperations = tryParseAndApplyOperations(response);
+          if (finalOperations.length > 0) {
+            applyOperationsToCell(finalOperations);
+            console.log(
+              `Applied ${finalOperations.length} edit_selection operations to lines ${startLine + 1}-${endLine + 1} in cell ${cellId}`
+            );
+          } else {
+            throw new Error('No valid operations found in response');
+          }
+        } catch (error) {
+          console.error(
+            'Failed to parse final edit_selection response:',
+            error
+          );
+          console.error('Raw response:', response);
+          alert(
+            `Error processing edit_selection: Invalid response format. Please try again.`
+          );
+          return false;
+        }
+      } else {
+        // For edit_full_cell, keep the original behavior
+        const targetCell = this.notebookTools.findCellByTrackingId(cellId);
+        if (!targetCell) {
+          throw new Error(`Could not find cell with ID ${cellId}`);
+        }
+
+        let accumulatedResponse = '';
+        let codeContent = '';
+        let isInCodeBlock = false;
+        let codeBlockStartPattern = /```(?:python|py)?\s*/i;
+        let codeBlockEndPattern = /```/;
+
+        const response = await chatService.sendEphemeralMessage(
+          contextMessage,
+          systemPrompt,
+          'claude-3-5-haiku-latest',
+          (textChunk: string) => {
+            accumulatedResponse += textChunk;
+
+            // Handle code extraction for streaming
+            if (!isInCodeBlock) {
+              const startMatch = accumulatedResponse.match(
+                codeBlockStartPattern
+              );
+              if (startMatch) {
+                isInCodeBlock = true;
+                // Remove everything up to and including the code block start (and any language specifier)
+                codeContent = accumulatedResponse.substring(
+                  accumulatedResponse.indexOf(startMatch[0]) +
+                    startMatch[0].length
+                );
+                // Remove any leading 'python' or 'py' line if present
+                codeContent = codeContent.replace(/^\s*(python|py)\s*\n?/i, '');
+              } else {
+                // Not in code block, just accumulate
+                codeContent = accumulatedResponse;
+              }
+            } else {
+              // Already in code block
+              if (codeBlockEndPattern.test(textChunk)) {
+                isInCodeBlock = false;
+                // Remove trailing ```
+                const endIndex = codeContent.lastIndexOf('```');
+                if (endIndex !== -1) {
+                  codeContent = codeContent.substring(0, endIndex);
+                }
+                // Remove any trailing 'python' or 'py' line if present
+                codeContent = codeContent.replace(/^\s*(python|py)\s*\n?/i, '');
+              } else {
+                codeContent += textChunk;
+              }
+            }
+
+            // Create progressive transformation effect
+            if (codeContent.includes('\n')) {
+              const newCodeLines = codeContent.split('\n');
+
+              // Progressive replacement: mix old and new code
+              const displayLines = [...cellLines];
+
+              // Replace lines progressively based on how much new content we have
+              const linesToReplace = Math.min(
+                newCodeLines.length,
+                cellLines.length
+              );
+
+              for (let i = 0; i < linesToReplace; i++) {
+                if (newCodeLines[i] !== undefined) {
+                  displayLines[i] = newCodeLines[i];
+                }
+              }
+
+              // If new code has more lines than original, add them
+              if (newCodeLines.length > cellLines.length) {
+                for (let i = cellLines.length; i < newCodeLines.length; i++) {
+                  if (newCodeLines[i] !== undefined) {
+                    displayLines.push(newCodeLines[i]);
+                  }
+                }
+              }
+
+              // Update cell with the progressive transformation
+              targetCell.cell.model.sharedModel.setSource(
+                displayLines.join('\n')
+              );
+            } else if (codeContent.trim() && !codeContent.includes('\n')) {
+              // Handle single line updates by replacing first line progressively
+              const displayLines = [...cellLines];
+              if (displayLines.length > 0) {
+                displayLines[0] = codeContent.trim();
+              } else {
+                displayLines.push(codeContent.trim());
+              }
+              targetCell.cell.model.sharedModel.setSource(
+                displayLines.join('\n')
+              );
+            }
+          }
+        );
+
+        // Final cleanup and application for full cell mode
+        let finalCode = codeContent.trim();
+        if (!finalCode && response.trim()) {
+          if (response.includes('```')) {
+            const codeMatch = response.match(
+              /```(?:python|py)?\s*([\s\S]*?)```/i
+            );
+            if (codeMatch) {
+              finalCode = codeMatch[1]
+                .replace(/^\s*(python|py)\s*\n?/i, '')
+                .trim();
+            }
+          } else {
+            finalCode = response.trim();
+          }
+        } else if (finalCode) {
+          // Remove any leading/trailing code block markers and 'python' lines
+          finalCode = finalCode
+            .replace(/^```(?:python|py)?\s*/i, '')
+            .replace(/```$/, '')
+            .replace(/^\s*(python|py)\s*\n?/i, '')
+            .trim();
+        }
+
+        // Always commit the last codeContent to the cell, even if not updated in the last chunk
+        if (finalCode) {
+          targetCell.cell.model.sharedModel.setSource(finalCode);
+          console.log(
+            `Applied edit_full_cell response to cell ${cellId} with progressive transformation`
+          );
+        }
+      }
+
+      console.log(
+        `Ephemeral ${modeLabel} request completed using Haiku model.`
+      );
+
+      return true;
+    } catch (error) {
+      const errorMessage =
+        error instanceof Error ? error.message : String(error);
+      console.error(`Failed to process: ${errorMessage}`, error);
+      return false;
+    }
+  }
+
   private async handlePromptSubmit(cell: Cell, prompt: string) {
     if (prompt.trim()) {
       const inputContainer = <HTMLInputElement>(
@@ -104,6 +526,20 @@ export class ContextCellHighlighter {
     }
   }
 
+  /**
+   * Helper function to calculate end line for selection
+   */
+  private calculateEndLine(end: any, cellLines: string[]) {
+    if (!end) return null;
+    if (end.line >= cellLines.length) {
+      return {
+        line: cellLines.length - 1,
+        column: cellLines[cellLines.length - 1]?.length || 0
+      };
+    }
+    return end;
+  }
+
   /**
    * Show diff view for a cell after AI modification
    */
@@ -1052,3 +1488,13 @@ const UNDO_ICON = `
         <path d="M21 17a9 9 0 0 0-9-9 9 9 0 0 0-6 2.3L3 13" stroke="var(--jp-ui-font-color3)" stroke-width="2" stroke-linecap="round" stroke-linejoin="round"/>
       </svg>
     `;
+
+interface EditOperation {
+  line: number;
+  action: 'KEEP' | 'MODIFY' | 'REMOVE' | 'INSERT';
+  content: string;
+}
+
+interface EditSelectionResponse {
+  operations: EditOperation[];
+}
