@@ -771,6 +771,14 @@ export class NotebookDiffManager {
     console.log('[NotebookDiffManager] All diffs cleared');
   }
 
+  /**
+   * Force revert all diffs and reject them
+   */
+  public rejectAndRevertDiffsImmediately(): void {
+    this.rejectAllDiffs();
+    this.handleRejectedDiffs();
+  }
+
   /**
    * Check if we should run code immediately after approval
    * This flag is set when "Approve All and Run" is clicked
