@@ -418,7 +418,7 @@ export class NotebookDiffTools {
    * Remove diff overlay from a cell
    * @param cell The cell to remove the diff overlay from
    */
-  removeDiffOverlay(cell: Cell): void {
+  static removeDiffOverlay(cell: Cell): void {
     try {
       const editorArea = cell.node.getElementsByClassName(
         'jp-InputArea-editor'
@@ -512,7 +512,7 @@ export class NotebookDiffTools {
       } = diffMeta;
 
       // First, remove the diff overlay
-      this.removeDiffOverlay(active);
+      NotebookDiffTools.removeDiffOverlay(active);
 
       // Apply or revert
       if (accept) {
